#!/usr/bin/env python3
# vim: fdm=indent ts=4

import h5py
import matplotlib.pyplot as plt
import numpy as np

import aa_generate_beacon as beacon
import lib

if __name__ == "__main__":
    from os import path
    import sys

    
    f_beacon_band = (49e-3,55e-3) #GHz
    allow_frequency_fitting = True
    read_frequency_from_file = True

    fname = "ZH_airshower/mysim.sry"

    ####
    fname_dir = path.dirname(fname)
    antennas_fname = path.join(fname_dir, beacon.antennas_fname)

    if not path.isfile(antennas_fname):
        print("Antenna file cannot be found, did you try generating a beacon?")
        sys.exit(1)

    # read in antennas
    with h5py.File(antennas_fname, 'a') as fp:
        if 'antennas' not in fp.keys():
            print("Antenna file corrupted? no antennas")
            sys.exit(1)

        group = fp['antennas']

        f_beacon = None
        if read_frequency_from_file and 'tx' in fp:
            tx = fp['tx']
            if 'f_beacon' in tx.attrs:
                f_beacon = tx.attrs['f_beacon']
            else:
                print("No frequency found in file.")
                sys.exit(2)
            f_beacon_estimate_band = 0.01*f_beacon

        elif allow_frequency_fitting:
            f_beacon_estimate_band = (f_beacon_band[1] - f_beacon_band[0])/2
            f_beacon = f_beacon_band[1] - f_beacon_estimate_band
        else:
            print("Not allowed to fit frequency and no tx group found in file.")
            sys.exit(2)

        N_antennas = len(group.keys())
        # just for funzies
        found_data = np.zeros((N_antennas, 3))

        # Determine frequency and phase
        for i, name in enumerate(group.keys()):
            ant_group = group[name]

            if 'traces' not in ant_group.keys():
                print(f"Antenna file corrupted? no 'traces' in {name}")
                sys.exit(1)

            traces = ant_group['traces']

            freqs, phases, amps = find_beacon_in_traces( 
                    traces[1:-1], traces[0],
                    f_beacon_estimate=f_beacon,
                    frequency_fit=allow_frequency_fitting,
                    f_beacon_estimate_band=f_beacon_estimate_band
                    )

            # only take Ex for now
            frequency = freqs[-1]
            phase = phases[-1]
            amplitude = amps[-1]

            print(frequency, phase, amplitude)

            ant_group.attrs['beacon_freq'] = frequency
            ant_group.attrs['beacon_phase'] = phase
            ant_group.attrs['beacon_amplitude'] = amplitude
            ant_group.attrs['beacon_orientation'] = 'Ex'

            found_data[i] = frequency, phase, amplitude

    # show histogram of found frequencies
    if True:
        if True or allow_frequency_fitting:
            fig, ax = plt.subplots()
            ax.set_xlabel("Frequency")
            ax.set_ylabel("Counts")
            ax.hist(found_data[:,0], bins='auto', density=False)

        if True:
            fig, ax = plt.subplots()
            ax.set_xlabel("Amplitudes")
            ax.set_ylabel("Counts")
            ax.hist(found_data[:,2], bins='auto', density=False)

        plt.show()
